package core.protocols.transport.socket.server;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;

import core.protocols.p2p.IDHT;
import core.protocols.p2p.Node;
import core.protocols.transport.IRequestHandler;
import core.synapse.Synapse;

/**
 * This class represent the socket transport layer
 * 
 * @author laurent.vanni@sophia.inria.fr - logNet team 2010 - INRIA
 *         Sophia-Antipolis - France
 * 
 */
public class SocketImpl extends AbstractServer {

	/** The request handler to use when a request is received */
	private IRequestHandler overlay = null;

	/**
	 * Default contructor
	 * 
	 * @see core.protocols.transport.socket.server.AbstractServer#AbstractServer(int,
	 *      int, String, int, int, int, IRequestHandler);
	 * @param port
	 * @param backlog
	 * @param requestHandlerClassName
	 * @param maxQueueLength
	 * @param minThreads
	 * @param maxThreads
	 * @param overlay
	 */
	public SocketImpl(int port, int backlog, String requestHandlerClassName,
			int maxQueueLength, int minThreads, int maxThreads,
			IRequestHandler overlay) {
		super(port, backlog, requestHandlerClassName, maxQueueLength,
				minThreads, maxThreads, overlay);
		this.overlay = overlay;
	}

	/**
	 * @see core.protocols.transport.ITransport#sendRequest(String, Node)
	 */
	public String sendRequest(String message, Node destination) {
		Socket socket = null;
		BufferedReader pin = null;
		PrintWriter pout = null;
		//if (!destination.getIp().equals(null) && !destination.getIp().equals(""))		
	//	System.out.println("In Socket implementation: sendRequest "+ message +" to "+destination.toString() );
		try {
			socket = new Socket(destination.getIp(), destination.getPort());			
		//	socket.setSoTimeout(TTL);
			
			pin = new BufferedReader(new InputStreamReader(socket
					.getInputStream()));
			pout = new PrintWriter(new BufferedWriter(new OutputStreamWriter(
					socket.getOutputStream())), true);
			pout.println(message);
			pout.flush();
			// WAIT FOR A RESPONSE
			String res = pin.readLine();
			// TTL = MAX_TTL;
			
			socket.close();
			pout.close();
			pin.close();

			return res;
		} catch (IOException e) {

			return "unreachable";
		}
	
	}	
	
}
