/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */
package core.synapse;

import static java.util.concurrent.TimeUnit.SECONDS;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;


import core.protocols.p2p.Node;
import core.protocols.transport.ITransport;
import core.synapse.Synapse.RequestInfo;

public class ReverseTable {
	private HashMap<String, ArrayList<Node>> reverseTable;	//netId -->[nodes]
	protected Random randomGenerator;
	protected ITransport transport;
	protected int interval = 180;
	ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
	
	public ReverseTable(ITransport transport)
	{
		reverseTable = new HashMap<String, ArrayList<Node>>();
		randomGenerator = new Random();
		this.transport = transport;
	//	refresh();
	}
	
	public synchronized void updateReverseTable(String connectedNetList, Node srcNode)
	{
		
	//	System.out.println("Received connectedList:"+connectedNetList);
			String[] nets = connectedNetList.split(";");
		
		for (int i=0; i< nets.length; i++)
		{	
	//		System.out.println("Update, net:"+nets[i]);
			if (reverseTable.containsKey(nets[i]))
			{	
		//		System.out.println("case1");				
				if (!reverseTable.get(nets[i]).contains(srcNode))
				{
					reverseTable.get(nets[i]).add(srcNode);
		//			print();
				}
			}	
			else
			{
				ArrayList tmp = new ArrayList();
				tmp.add(srcNode);
				reverseTable.put(nets[i], tmp);
//				System.out.println("case2");
	//			print();
			}
		}
	}
	
	public synchronized ArrayList<Node> getRemoteSynapses(ArrayList<String> nets)
	{
		ArrayList<Node> dests = new ArrayList<Node>();
		for (int i=0; i< nets.size(); i++)
		{	
			String netId = (String)nets.get(i);
			int nodeNumbers = reverseTable.get(netId).size();
			if (nodeNumbers > 0)
			{	
				//get a random Synapse
				int nodeIndex = randomGenerator.nextInt(nodeNumbers);
				Node dest = reverseTable.get(netId).get(nodeIndex);		
				dests.add(dest);
			}	
		}
		return dests;
	}
	
	
	//get netIds that do not process requestInfo
	//update requestInfo
	public synchronized ArrayList<String> getNets(RequestInfo requestInfo)
	{
		ArrayList<String> result = new ArrayList<String>();
		
		if (reverseTable.keySet().size()>0)
		{
			String processedNetList = requestInfo.getProcessedNetList();
			String[] list = processedNetList.split(";");
			List netList = Arrays.asList(list);
			Iterator iter = reverseTable.keySet().iterator();
			while (iter.hasNext())
			{
				String netId = (String)iter.next();
				if (!netList.contains(netId) && reverseTable.get(netId).size() > 0)
				{
					result.add(netId);
				}
			}
		}
		
		return result;
	}
	
	
	public void refresh()
	{
//		System.out.println("Call refresh() function");
		scheduler.schedule(new FixReverseTable(), interval, SECONDS);
	}
	
	
	public void print()
	{
		if (reverseTable.keySet().size()>0)
		{
			Iterator iter = reverseTable.keySet().iterator();
			while (iter.hasNext())
			{
				String netId = (String)iter.next();
	//			System.out.println("reverseTable, netId:"+netId
	//					+","+reverseTable.get(netId).toString());
			}
		}	
	}
		
	public class FixReverseTable implements Runnable 
	{
		public void run()
		{
			
		if (reverseTable.keySet().size()>0)
		{
	//		System.out.println("Run FixReverseTable()");
			ArrayList<String> emptyNets = new ArrayList<String>();
			Iterator iter = reverseTable.keySet().iterator();
			while (iter.hasNext())
			{
				String netId = (String)iter.next();
				int nodeNumbers = reverseTable.get(netId).size();
				if (nodeNumbers > 0)
				{	
					for (int i = nodeNumbers-1; i>-1; i--)
					{
						Node dest = reverseTable.get(netId).get(i);		
	
						//Message: code
						int code = ISynapse.SYNAPSEPING;	
						String message = Integer.toString(code);	
						String result = transport.sendRequest(message, dest);
						
				//		System.out.println("Send PING message to "+dest.toString()
					//			+" receive result: "+result);

						if ((result == null) || result.equals("null") 
								|| result.equals("unreachable"))
						{
							reverseTable.get(netId).remove(i);
						}
					}					
					if (reverseTable.get(netId).size() == 0)
						emptyNets.add(netId);
				}
				else 
					emptyNets.add(netId);	
			}
			
			if (emptyNets.size() > 0)
				for (int i=0; i< emptyNets.size(); i++)
					reverseTable.remove(emptyNets.get(i));
		}
	}	
}

}
