/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */

package core.synapse;
import static java.util.concurrent.TimeUnit.SECONDS;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.Timer;
import java.util.TimerTask;
import core.protocols.p2p.IDHT;
import core.protocols.p2p.Node;
import core.protocols.transport.ITransport;
import core.protocols.transport.socket.request.RequestHandler;
import core.protocols.transport.socket.server.SocketImpl;
import core.tools.HashFunction;
import experiments.current.MessageCodes;
import experiments.current.node.chord.ChordNode;
import experiments.current.node.kademlia.KadNode;

public class Synapse implements ISynapse, IDHT {

	/** networks where the synapse is registered */
	protected List<IDHT> networks;
	/** information to construct an unique identifier */
	private static SimpleDateFormat formater = new SimpleDateFormat(
			"dd/MM/yy_H:mm:ss");
	protected static String time = formater.format(new Date());
	/** Synapse network identifier */
	public String identifier;
	protected HashFunction h;
	protected int TTL;
	protected boolean goodDeal;
	protected Random randomGenerator;
	protected Node node;
	protected Node manager;
	protected ITransport transport;
	protected Boolean alive = true;	
	protected int messageNumber = 0;
	protected Configuration conf;
	protected long startTime;
	protected long endTime;
	protected int timeout = 90000; //milliseconds
	private ConcurrentHashMap<Integer, ArrayList<Integer>> reqIdTable; //1 hKey --> n reqId	
	private HashMap<Integer, RequestInfo> reqInfoTable; //reqId --> reqInfo(cleanKey, TTL, )
	private HashMap<Integer, ArrayList<String>> resultTable; 	//reqId--> [results]	
	
	//private Map<Integer, ArrayList<String>> cacheResultTable; 	//cleanKey--> [results]
	
	private HashMap<Integer, Node> forwardingTable; 	//reqId--> [srcNodes]
	private ArrayList<Integer> processedReqIdList; 	//[reqId]: already processed reqId
	private HashMap<String, ArrayList<Node>> reverseTable;	//netId -->[nodes]
	private HashMap timeoutList;     // keeps track of timeout events
	private Timer timer;
	ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
	
	public Synapse(String ip, int port, String identifier, String manager) {
		//System.out.println("In Synapse: create: "+ip+":"+port );
		this.identifier = identifier;
		this.h = new HashFunction(identifier);
		int id = h.SHA1ToInt(ip + port + time);
		this.manager = new Node(manager);
		networks = new ArrayList<IDHT>();
		timer = new Timer(true);
		timeoutList = new HashMap();
		reqIdTable = new ConcurrentHashMap<Integer, ArrayList<Integer>>();
		reqInfoTable = new HashMap<Integer, RequestInfo>();
		resultTable = new HashMap<Integer, ArrayList<String>>();
		forwardingTable = new HashMap<Integer, Node>();
		reverseTable = new HashMap<String, ArrayList<Node>>();
		processedReqIdList = new ArrayList<Integer>();
		randomGenerator = new Random();
		this.node = new Node(ip, id, port);
		String filename = "synapse.conf";
		conf = new Configuration(filename);
		
		TTL = conf.TTL;
		goodDeal = conf.goodDeal;
		// The transport layer
		this.transport = (ITransport) new SocketImpl(port, 10, RequestHandler.class
				.getName(), 10, 1, 50, this);
		((SocketImpl) transport).launchServer();
		startTime = System.currentTimeMillis();
		
	}
	
	//for initiator node
	public String get(String key)
	{
		int count =0;
		int reqId = Math.abs(randomGenerator.nextInt()); 	//tag the reqId
		ArrayList result = new ArrayList<String>();		//initialize the result entry  
		resultTable.put(reqId, result);
		String processedNetList = "";		//initialize alreadySentNetList		
		//initialize reqInfo
		RequestInfo requestInfo = new RequestInfo(key, this.TTL, processedNetList);
		//mark arealdy process this reqId
		processedReqIdList.add(reqId);

		
		get(reqId, count, requestInfo);
		
		// WAIT FOR COLLECTING VALUES
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		int resultNumber = resultTable.get(reqId).size();
		//System.out.println("result number:"+resultNumber);
		
		//return first result to manager
		String res = "null";
		if (resultNumber > 0) 
			res = resultTable.get(reqId).get(0);
		return res;
		/*		
		//Read other result
		String res = "";
		if (resultNumber > 0)
		{	
		for (int i=0; i< resultNumber; i++) 
		  res =  resultTable.get(reqId).get(i) +";"+ res;
			res = res.substring(0, res.length() - 1);
			System.out.println("result:"+res);
		}
		 */			
		
	}
	
	
	
	//any node --> get request and write to the result table or send back to originator node
	public void get(int reqId, int count, RequestInfo requestInfo) {
		//	System.out.println("In Synapse get(reqId) "+reqId);
		//schedule the timeout
		TimeoutRequest timeoutRequest = new TimeoutRequest(reqId);
		timer.schedule(timeoutRequest, timeout);
		timeoutList.put(reqId, timeoutRequest);

		if (requestInfo.getTTL() > 0)
		{
			ArrayList<IDHT> connectedNets;
			ArrayList<String> remoteNets;
			//System.out.println("1.requestInfo.getProcessedNetList(): "+requestInfo.getProcessedNetList());
			
			//get connected net to send GetRequest and update processedNetList
			connectedNets = getConnectedCandidateNets(requestInfo);
			
			//System.out.println("2. requestInfo.getProcessedNetList(): "+requestInfo.getProcessedNetList());
			//get remote net to send GetRequest via other Synapse and update processedNetList 
			remoteNets = getRemoteCandidateNets(requestInfo);
			
			//System.out.println("3. requestInfo.getProcessedNetList(): "+requestInfo.getProcessedNetList());
			
			if ((connectedNets.size()> 0) || (remoteNets.size()>0))
			{	
				
				//create new requestInfo
				int newTTL = requestInfo.getTTL() -1; 	
				requestInfo.setTTL(newTTL);
				reqInfoTable.put(reqId, requestInfo);
								
				//send Get request to connected overlays
				if (connectedNets.size()> 0)
					sendGetRequestToConnectedNets(reqId, count, requestInfo, connectedNets);		
				
				//send Get request to remote overlays
				if (remoteNets.size()>0)
				sendGetRequestToRemoteNets(reqId, count, requestInfo, remoteNets);		
			}
		}
	}		
	

	
	public void sendGetRequestToRemoteNets(int reqId, int count, RequestInfo requestInfo, ArrayList<String> nets)
	{
		for (int i=0; i< nets.size(); i++)
		{	
			String netId = (String)nets.get(i);
			int nodeNumbers = reverseTable.get(netId).size();
			if (nodeNumbers > 0)
			{	
				//get a random Synapse
				int nodeIndex = randomGenerator.nextInt(nodeNumbers);
				Node dest = reverseTable.get(netId).get(nodeIndex);		
				
				//Message: code, reqId, requestInfo, connectedNetList,thisNode
				String localNetList = getConnectedNetList();
				int code = ISynapse.SYNAPSEGET;	
				count+=1;
				
			//	System.out.println("countsynapse,send getrequest to remote Syn"+count+":"+reqId);
				String message = code+","+reqId +","+ count +","+requestInfo.toString()+","
				+localNetList+","+getThisNode().toString();
				//System.out.println("Send request to netID: "+netId);
				//send GetRequest to dest Synapse
				//(new sendRequest(message, dest)).start();	
				sendRequest(message, dest);
			}	
		}
	}
	
	public void sendGetRequestToConnectedNets(int reqId, int count, RequestInfo requestInfo, ArrayList<IDHT> nets)
	{

		String key = requestInfo.getCleanKey();
						
		// sent Get request to all connected nets		
		for (int i = 0; i < nets.size(); i++) {
			IDHT o = nets.get(i);
			if (((o != null) )&&(!o.getIdentifier().equals(""))) {
				int hkey = o.keyToH(key);
			//add entry to reqIdTable --> concurrentHashMap	
				if (!reqIdTable.containsKey(hkey)){
					ArrayList result;
					ArrayList<Integer> tmp = new ArrayList<Integer>();
					tmp.add(reqId);
					result = reqIdTable.putIfAbsent(hkey, tmp);
					if (!(result == null)) reqIdTable.get(hkey).add(reqId);
				}
				else 
					reqIdTable.get(hkey).add(reqId);
				
				String shkey = String.valueOf(hkey);
				new Thread(new Get(shkey, count, o, reqId, this)).start();
			}
			
		}			

	}
	
	//get list of connectednets to send the Get request  
	//and update the processedNetList
	public ArrayList<IDHT> getConnectedCandidateNets(RequestInfo requestInfo)
	{
		ArrayList<IDHT> nets = new ArrayList<IDHT>();
		String processedNetList = requestInfo.getProcessedNetList();
		//get connected net to send the get request, this is initiator node of Get request
		if (processedNetList.equals("")) 
			nets = (ArrayList)networks;
		else 
		{
			String[] list = processedNetList.split(";");
			List processedList = Arrays.asList(list);		
			for (int i = 0; i < networks.size(); i++) {
				IDHT o = networks.get(i);
				if (((o != null) )&&(!o.getIdentifier().equals(""))
						&&(!processedList.contains(o.getIdentifier())))
					nets.add(o);
			}	
		}
		
		//update the processedNetList
		if (nets.size() > 0)
		{	
			for (int i = 0; i < nets.size(); i++) {
				IDHT o = nets.get(i);
				if (((o != null) )&&(!o.getIdentifier().equals(""))) {
					processedNetList = o.getIdentifier()+ ";" + processedNetList;
				}
			}
			if (processedNetList.charAt(processedNetList.length()-1) == ';')
				processedNetList = processedNetList.substring(0, processedNetList.length()-1);
			requestInfo.setProcessedNetList(processedNetList);
		}
		return nets;
	}
	
	//get list of remote nets to send the Get request via Synapse 
	//and update the processedNetList
	public ArrayList<String> getRemoteCandidateNets(RequestInfo requestInfo)
	{
		ArrayList<String> result = new ArrayList<String>();
		
		if (reverseTable.keySet().size()>0)
		{
			String processedNetList = requestInfo.getProcessedNetList();
			String[] list = processedNetList.split(";");
			List netList = Arrays.asList(list);
			Iterator iter = reverseTable.keySet().iterator();
			while (iter.hasNext())
			{
				String netId = (String)iter.next();
				if (!netList.contains(netId))
				{
					result.add(netId);
					processedNetList = netId + processedNetList;
				}
			}
			requestInfo.setProcessedNetList(processedNetList);	
		}	
		return result;
	}
	

	/**
	 * InnerClass , define a threaded get request
	 */
	private class Get implements Runnable {
		private String hKey;
		private int count;
		private Synapse s;
		private IDHT o;
		private int reqId;
		
		public Get(String hKey, int count, IDHT o, int reqId, Synapse s) {
			this.hKey = hKey;
			this.count = count;
			this.s = s;
			this.o = o;
			this.reqId = reqId;
		}

		public void run() {
			String result="null";
			try {
		//System.out.println("Synapse connectednet Key "+hKey+ "count="+count);
				result = o.get(hKey, count);
		//System.out.println("Synapse connectednet Key "+hKey+" result: "+result);

			} catch (NullPointerException e) {}
					
			String message; 
			if (!(result == null) && !result.substring(0,4).equals("null") && !result.equals("unreachable"))
			{			
				if (resultTable.containsKey(reqId)) //this is initiator node
				{
				//	System.out.println("add result: "+result+" to table corresponding to: "+ reqId);
					resultTable.get(reqId).add(result);
				}
	
				else //this is forwarding node
				{
					if (forwardingTable.containsKey(reqId))
					{
						//code + reqId + result + count + srcNode
						message = ISynapse.SYNAPSERESULT+"," + reqId +","+result+","+getThisNode().toString();
						//(new sendRequest(message, forwardingTable.get(reqId))).start();
						//System.out.println("forward result "+result+" to "+forwardingTable.get(reqId));
						sendRequest(message, forwardingTable.get(reqId));
					}	
				}
			}
		}
	}

	
//receive the GET request from normal overlay 
public void getReceiver(String hkey, int count, Node srcNode)
{
	if (srcNode != null) 
	{	
		
		//create Synapse offer message
		int code = ISynapse.SYNAPSEOFFER;	
		String connectedNetList = getConnectedNetList();
		count+=1;
	//	System.out.println("countsynapse,GetReceiver "+count+":"+hkey);
		//0code, 1hkey, 2count, 3connectedlist, 4srcNode
		String message = code +","+ hkey +","+ count+","+connectedNetList +"," +getThisNode().toString();
//		System.out.println("send Synapse offer to srcNode:"+srcNode.toString());
		//(new sendRequest(message, srcNode)).start();
		sendRequest(message, srcNode);
	}
}
	



//handle synapseOffer message
public void synapseOfferReceiver(int hkey, int count, String connectedNetList, Node srcNode)
{
	String message = "";
	String cleanKey = "";
	int ttl;
	int reqId;
	//System.out.println("receive Synapse offer from Synapse: "+srcNode.toString());
	//update reverseTable
	updateReverseTable(connectedNetList, srcNode);
		
	//create response message
	if (reqIdTable.containsKey(hkey))
	{
		ArrayList reqIds = reqIdTable.get(hkey); 
		for (int i=0; i< reqIds.size(); i++)
		{	
			String localNetList ="";
			reqId = (Integer)reqIds.get(i);
			//System.out.println("In synapseOfferReceiver, reqId:"+reqId);
			
			RequestInfo requestInfo = reqInfoTable.get(reqId);
			
			if (!(requestInfo==null))
			{	
				count+=1;
				//System.out.println("countsynapse,SynapseOfferReceiver "+count+":"+hkey);
				//System.out.println("RequestInfo:"+requestInfo);
				//System.out.println("In synapseOfferReceiver, reqInfo:"+requestInfo.toString());
				localNetList = getConnectedNetList();
				int code = ISynapse.SYNAPSEGET;
				//System.out.println("receive Synapse offer");
				//0code,1reqId, 2count, 3reqInfo,4localNetList,5srcNode
				message = code+","+reqId+","+count+","+requestInfo.toString()+","+
					localNetList+","+getThisNode().toString();
				//System.out.println("after receive Synapse offer, send message:"+message);
			//	System.out.println("after receive Synapse offer, send the SynapseGet requestId"+reqId+":"+requestInfo.toString()+":"+srcNode.toString());
						//(new sendRequest(message, srcNode)).start();
				sendRequest(message, srcNode);
			
			}
		}
	}
		
	//return: String cleanKey, String connectedNetList, String TTL, Node
}


//receive the GET request from other Synapse
//message = code,reqId,count, reqInfo,localNetList,srcNode;
public void synapseGetReceiver(int reqId, int count, RequestInfo requestInfo, 
		String localNetList, Node srcNode)
{
	//System.out.println("Synapse receive Get request from other Synapse:"+srcNode);
	updateReverseTable(localNetList, srcNode);
	
	//has not processed this request
	
	if (!processedReqIdList.contains(reqId))
	{	
		//mark arealdy process this reqId
	//	System.out.println("Synapse receive Get request from other Synapse, not yet processed");
		processedReqIdList.add(reqId);
		forwardingTable.put(reqId, srcNode);
		//System.out.println("countsynapse,SynapseGet receiver "+count+":"+reqId);
		get(reqId, count, requestInfo); //find and return the result
	}
	
}
	

public void synapseResultReceiver(int reqId, String result, int count)
{
	String message;
	//System.out.println("Synapse receive result from other Synapse: "+ result+" for reqId: "+reqId);
	if (!(result ==null) && !result.equals("null"))
	{	
		count++;
		//System.out.println("countsynapse,Synapse result receiver "+count+":"+reqId);
		if (resultTable.containsKey(reqId))//this is initiator node
		{
			result = result+","+count;
			resultTable.get(reqId).add(result);
			
		}	
		else //this is forwarding node
		{
			if (forwardingTable.containsKey(reqId))	
			{
				//System.out.println("Synapse forwarding receive result: "+ result+" for reqId: "+reqId);
				message = ISynapse.SYNAPSERESULT+"," + reqId +","+result+","+ count +","+getThisNode().toString();
				sendRequest(message, forwardingTable.get(reqId));
			}	
		}	
	}
}	

	public void sendRequest(String message, Node destination) {
		messageNumber++;
		transport.sendRequest(message, destination);
	}
	

	/**
	 * @see core.protocols.transport.IRequestHandler#handleRequest(String)
	 *GetRequest: (0code, 1reqId, 2count, 3reqInfo[cleanKey, TTL, alreadyReceivedNetId], 
	 *4connectedNetList, 5,6,7srcNode)
		SynapseOffer: (0code, 1hKey, 2connectedNets, 3srcNode)
		SynapseResponse: code, reqId, result, srcNode
	 *
	 */
public String handleRequest(String code) {
	//System.out.println("Synapse receive: "+code);
		Node srcNode;
		String result = "";
		String[] args = code.split(",");
		
		if (args[0].equals(String.valueOf(ISynapse.SYNAPSEGET))){
			
			srcNode = new Node(args[5], Integer.parseInt(args[7]));
			RequestInfo requestInfo = new RequestInfo(args[3]);
		//	System.out.println("Receive SynapseGet from overlays "+srcNode.toString());
			synapseGetReceiver(Integer.parseInt(args[1]),Integer.parseInt(args[2]),
					requestInfo, args[4], srcNode);
		
		}
		else if (args[0].equals(String.valueOf(ISynapse.SYNAPSERESULT))){
			synapseResultReceiver(Integer.parseInt(args[1]), args[2], Integer.parseInt(args[3]));
		} else if (args[0].equals(String.valueOf(MessageCodes.DELETE))){
			result = deleteReceiver();
		} else if (args[0].equals(String.valueOf(MessageCodes.MANAGERCOL))){
			result = colStatsReceiver();
		}
		else if (args[0].equals(String.valueOf(MessageCodes.MANAGERGET))){
			//System.out.println("SYnapse, receive lookup from manager "+code);
			result = get(args[1]);
		}
		
		return result;
	}

	//convert connectedNetList to String
	public String getConnectedNetList()
	{
		String connectedNetList = ""; 
		int size = getNetworks().size();
		for (int i=0; i< size; i++)
		connectedNetList = connectedNetList + ";" + getNetworks().get(i).getIdentifier();
		connectedNetList = connectedNetList.substring(1, connectedNetList.length());
		return connectedNetList;
	}


	//update reverseTable
	public void updateReverseTable(String connectedNetList, Node srcNode)
	{

			String[] nets = connectedNetList.split(";");
		
		for (int i=0; i< nets.length; i++)
		{			
			if (reverseTable.containsKey(nets[i]))
				if (!reverseTable.get(nets[i]).contains(srcNode))
					reverseTable.get(nets[i]).add(srcNode);
			else
				{
					ArrayList tmp = new ArrayList();
					tmp.add(srcNode);
					reverseTable.put(nets[i], tmp);
				}
		}

	}
	
	
	
	//print all about the synapse
	@Override
	public String toString() {
		String res = identifier + " on " + getThisNode().getIp() + ":"
				+ getThisNode().getPort() + "\n" + super.toString();
		res+= "Tables on Synapse";
		
		if (reqIdTable.size() > 0) 
			res = res + "\n reqIdTable:\n"+reqIdTable.toString();
		if (resultTable.size() > 0) 
			res = res + "\n resultTable:\n"+resultTable.toString();
		if (forwardingTable.size() > 0) 
			res = res + "\n forwardingTable:\n"+forwardingTable.toString();	
		if (reverseTable.size() > 0) 
			res = res + "\n reverseTable:\n"+reverseTable.toString();		
				
		res += "\n\n Networks:\n";
		for (IDHT o : networks) {
			res += "\n\n" + o.toString();
		}
		return res;
	}


	/**
	 * @see core.protocols.p2p.IDHT#getTransport()
	 */
	public ITransport getTransport() {
		return transport;
	}

	/**
	 * 
	 * @return the list of the networks where the synapse is connected
	 */
	public List<IDHT> getNetworks() {
		return networks;
	}

	/**
	 * @param key
	 * @return the hash value of the key
	 */
	public int keyToH(String key) { // A CHANGER!
		try {
			return Integer.parseInt(key);
		} catch (NumberFormatException e) {
			return h.SHA1ToInt(key);
		}
	}

	// message:(plugin)[netId:nodeInfo] synapse:messageNumber:duration, [chord:messageNumber:duration,
	// kad:messageNumber:duration]
	public String deleteReceiver()
	{
		//	System.out.println("Synapse receive DELETE message");
		
		String pluginMessage;
		endTime = System.currentTimeMillis();
		int duration = Math.round((endTime - startTime)/1000);
		
		String message ="synapse,"+messageNumber+","+String.valueOf(duration);
		
		for (IDHT o : networks) {
			pluginMessage = o.deleteReceiver();
			message = message + ":" + pluginMessage;
		}
		
		scheduler.schedule(new StopServer(), 12, SECONDS);
		//System.out.println("In SYnapse number of message:"+message);
		return message;
	}

	public String colStatsReceiver()
	{
		
		//System.out.println("Synapse receive COLSTATS message");
		endTime = System.currentTimeMillis();
		int duration = Math.round((endTime - startTime)/1000);
		String message ="synapse,"+messageNumber+","+String.valueOf(duration);
		scheduler.schedule(new StopServer(), 10, SECONDS);
		//System.out.println("In SYnapse number of message:"+message);
		return message;
	}

	
	
	public void kill() {
	}

	
	@Override
	public Node getThisNode() {
		// TODO Auto-generated method stub
		return node;
	}



	@Override
	public void put(String key, String value) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void invite() {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void join(String host, int port) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public String getIdentifier() {
		
		return identifier;
	}

class RequestInfo
{
	
	protected String cleanKey;
	protected int TTL;
	protected String processedNetList;
	public RequestInfo(String cleanKey, int TTL, String processedNetList)
	{
		this.cleanKey = cleanKey;
		this.TTL = TTL;
		this.processedNetList = processedNetList;
	}
	
	public RequestInfo(String args)
	{
		String[] arg = args.split(":");
		this.cleanKey = arg[0];
		this.TTL = Integer.parseInt(arg[1]);
		this.processedNetList = arg[2];
	}
	
	public String getCleanKey()
	{
		return cleanKey;
	}
	public int getTTL()
	{
		return TTL;
	}
	
	public String getProcessedNetList()
	{
		return processedNetList;
	}
	
	public String toString()
	{
		return cleanKey+":"+ TTL + ":" + processedNetList; 
	}
	
	public void setTTL(int newTTL)
	{
		 TTL = newTTL;
	}
	
	public void setProcessedNetList(String newProcessedNetList)
	{
		processedNetList = newProcessedNetList;
	}		
}

class StopServer implements Runnable
{
	public void run()
	{
		//System.out.println("In Synapse :"+node.toString()+" killed");
		((SocketImpl)transport).stopServer();
		System.exit(0);
	}
}



class sendRequest extends Thread{
String message;
Node destination;
public sendRequest (String message, Node destination) {
	this.message = message;
	this.destination = destination;
}
public void run()
{
//	System.out.println("In Synapse, send message:"+ message +"dest:"+destination.toString());
	transport.sendRequest(message, destination);
}	
	//return res;
}


class TimeoutRequest extends TimerTask {
	int reqId;
	public TimeoutRequest(int reqId)
	{
		this.reqId = reqId;
	}
	public void run(){
		
		if (reqInfoTable.containsKey(reqId))
		{
			reqInfoTable.remove(reqId);
			//System.out.println("remove reqId"+reqId+"from reqInfoTable");
		}
		
		if (resultTable.containsKey(reqId))
		{
			resultTable.remove(reqId);
			//System.out.println("remove reqId"+reqId+"from result table");
		}
		
		if (forwardingTable.containsKey(reqId))
		{
			forwardingTable.remove(reqId);
			//System.out.println("remove reqId"+reqId+"from forwarding table");
		}
		
		if (processedReqIdList.contains(reqId))
		{	
			processedReqIdList.remove((Integer)reqId);
		//	System.out.println("remove reqId"+reqId+"from processedReqIdList");
		}	
		for (Map.Entry<Integer, ArrayList<Integer>> entry : reqIdTable.entrySet())
		{
			
			ArrayList<Integer> reqIdList = entry.getValue();
			
			if (reqIdList.contains(reqId))
			{
				reqIdList.remove((Integer)reqId);
				//System.out.println("remove reqId"+reqId+"from reqIdtable");
			}	
			if (reqIdList.size() == 0)
			{	
				//System.out.println("remove hkey"+entry.getKey()+"from reqIdtable");
				reqIdTable.remove(entry.getKey());
			}	
			
		}	
	
		if (timeoutList.containsKey(reqId))
		{	
			timeoutList.remove(reqId);
			//System.out.println("remove reqId"+reqId+"from timeout list");
		}	
	}

}


@Override
public String get(String key, int count) {
	// TODO Auto-generated method stub
	return null;
}

}

/*algorithm: 

SynapseGetRequest: (reqId, reqInfo[cleanKey, TTL, alreadyReceivedNetId], connectedNetList, srcNode)
SynapseOffer: (hKey, connectedNets, srcNode)
SynapseResponse: (reqId, result)

- initiator node receive SYnapseGet from application/manager
	+ tag reqId, initialize reqInfo, add resultTable 
	+ send GetRequest to connected overlays
	+ send GetRequest to not connected overlays via known Synapse --> SynapseGetRequest
	+ schedule to timeout entry corresponding to reqId in the resultTable

- receive SynapseGetRequest from other Synapse
	- if reqId is not processed and if there is netID has not received reqId
		  + add to forwardingTable 
		  + modify reqInfo
		  + send GetRequest(reqId, cleanKey, TTL) to connected overlays
		  + send GetRequest(reqId, cleanKey, TTL) to not connected overlays 
		  via known Synapse
		  + schedule to timeout entry corresponding to reqId in the forwardingTable
		

- receive GETrequest from plugins
		+ send SynapseOFFER  
	
- receive SynapseOFFER(hKey) from other Synapse 
		+ get reqId associated to hKey
		+ send GetRequests associated with these reqId to offerSynapse
	 	+ update reverseTable if offerNode connected to other overlay

- receive Response(reqId) from connected overlay or from other Synapses: 
		+ if this Synapse is the initiator of reqId, 
			- if the first request, send to manager node, add to resultTable
			- if not the first, add to the resultTable
			
		- if no(forwarding node): 
			- forward to srcNode corresponding to reqId in the pedingTable
			(reqId --> cleanKey--> hKey --> other reqId --> send the 
			result to srcNode corresponding to all reqId)  

*/