package experiments.current;

import experiments.current.tracker.core.*;
//import core.ITracker;
import core.protocols.p2p.IDHT;
import core.protocols.p2p.Node;
import core.synapse.*;
import core.protocols.p2p.chord.AbstractChord;
import core.tools.InfoConsole;
import experiments.current.node.chord.ChordNode;
import experiments.current.node.kademlia.KadNode;
import experiments.current.synapse.plugin.chord.ChordNodePlugin;
import experiments.current.synapse.plugin.kademlia.KadNodePlugin;

/**
 * This a very simple User Interface to allow to create Synapse, node and
 * plugin. And test the DHT functionality
 * 
 * @author laurent.vanni@sophia.inria.fr - logNet team 2010 - INRIA
 *         Sophia-Antipolis - France
 * 
 * Modified by giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */
public class Create {
	
	private static enum NodeType {
		CHORD, CHORDPLUGIN, KAD, KADPLUGIN, SYNAPSE
	}

	/**
	 * Instantiate a node of the type NodeType
	 * 
	 * @param protocol
	 * @param id of overlay
	 * @param t
	 * @param s
	 * @return the node created
	 */
	public static IDHT getNode(String protocol, String ip, int port, String id, String bootstrap, 
			String manager, int putDelay, NodeType t, Synapse s) {
		//System.out.println("create Node");
		IDHT overlay = null;
		//String ip = InfoConsole.getIp();
		if (protocol.equals("chord") || protocol.equals("CHORD")) {
			
			AbstractChord node = null;
			switch (t) {
			case CHORD:
				node = new ChordNode(ip, port, id, manager, putDelay);
				break;
			case CHORDPLUGIN:
				node = new ChordNodePlugin(ip, port, id, manager, putDelay, s);
				break;
			}
			overlay = node;
			
		} else if (protocol.equals("synapse") || protocol.equals("SYNAPSE")) {
			
			overlay = new Synapse(ip, port, "synapse", manager);			
			
		} else if (protocol.equals("kad") || protocol.equals("KAD")) {
			switch (t) {
			case KAD:
				overlay = new KadNode(ip, port, id, manager, putDelay);
				break;
			case KADPLUGIN:
				overlay = new KadNodePlugin(ip, port, id, manager, s, putDelay);
				break;
			}
		}
	
		// CONNECT ON TRACKER	
		Node tracker = new Node(manager);
		//debug
		//System.out.println("\n Manger: " + tracker.toString());
		
	
				
		if (overlay instanceof Synapse)
		{
			overlay.getTransport().sendRequest(
					ITracker.ADDNODE + "," + overlay.getIdentifier() + ","
					+ overlay.getThisNode() + "," + overlay.getTransport().getPort(), tracker);
			
		}
		else 
		{
			
			Node joinPoint;	
			if (!bootstrap.equals("null"))
			{	
				joinPoint = new Node(bootstrap);
				//debug
				//System.out.println("\n Bootstrap: " + joinPoint.toString());
				overlay.getTransport().sendRequest(	MessageCodes.ADDNODE + "," + 
				overlay.getIdentifier() + "," + overlay.getThisNode() + "," 
				+ overlay.getTransport().getPort(), tracker);
				
				//join
				overlay.join(joinPoint.getIp(), joinPoint.getPort());
			}
			else
			{	
				//System.out.println("send GETCONNECTION request to manager");
				String trackerResponse = overlay.getTransport().sendRequest
				(MessageCodes.GETCONNECTION + "," + overlay.getIdentifier(), tracker);
				
				//debug
			//	System.out.println("\n Tracker response: " + trackerResponse);		
	
	//			System.out.println("send ADDNODE request to manager");
				overlay.getTransport().sendRequest(MessageCodes.ADDNODE + "," + 
				overlay.getIdentifier() + "," + overlay.getThisNode() + "," 
				+ overlay.getTransport().getPort(), tracker);
				
				if (!trackerResponse.equals("unreachable") && !trackerResponse.equals("null")) {
					joinPoint = new Node(trackerResponse);
					overlay.join(joinPoint.getIp(), joinPoint.getPort());
				
				} else {
					System.out.println("Warning: node launched in standalone mode\n");
				}
			}
		}
		return overlay;
	}
	


	//Main() function, args 
	//node 	 1chord/kad 2machineIp 3port 4netid  5bootstrap 6manager 7putDelay
	//synapse 1machineIp 2port 3manager 4-a 5chord/kad 6port 7netid  8bootstrap 9putDelay

	public static void main(String[] args) {
		try {
			IDHT overlay = null;
			if (args[0].equals("node")) {
				if(args[1].equals("chord") || args[1].equals("CHORD")) {
					overlay = getNode(args[1], args[2], Integer.parseInt(args[3]), 
							args[4],  args[5], args[6], Integer.parseInt(args[7]), NodeType.CHORD,null);
				} else if  (args[1].equals("kad") || args[1].equals("KAD"))	{ 
					overlay = getNode(args[1], args[2], Integer.parseInt(args[3]), 
							args[4],  args[5], args[6], Integer.parseInt(args[7]), NodeType.KAD, null);
				}  
			} else if (args[0].equals("synapse")) {
				
				String manager = args[3];
				String ip = args[1];
				Synapse synapse = (Synapse) getNode("synapse", ip, Integer.parseInt(args[2]),
						"SYNAPSE", null, manager, 0, NodeType.SYNAPSE, null);
				
				
				
				for (int i = 4; i < args.length; i += 6) {
					if (args[i].equals("-a")) {
						
						if (args[i + 1].equals("chord") || args[i+1].equals("CHORD")) {
							IDHT o = getNode("chord", ip, Integer.parseInt(args[i + 2]), 
									args[i + 3], args[i + 4], manager, Integer.parseInt(args[i + 5]), 
									NodeType.CHORDPLUGIN, synapse);
						
							synapse.getNetworks().add(o);
						} else if (args[i + 1].equals("kad") || args[i+1].equals("KAD")) {
							IDHT o = getNode("kad", ip, Integer.parseInt(args[i + 2]), 
									args[i + 3], args[i + 4], manager, Integer.parseInt(args[i + 5]),
									NodeType.KADPLUGIN, synapse);
						
							synapse.getNetworks().add(o);
						}
					}
				}
				overlay = synapse;
			}
		

		} catch (ArrayIndexOutOfBoundsException e) {
			System.out.println(e);
		}
		
	}
}

