package experiments.current.node.kademlia;

import static java.util.concurrent.TimeUnit.SECONDS;

import java.io.IOException;
import java.math.BigInteger;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import org.planx.xmlstore.routing.Identifier;
import org.planx.xmlstore.routing.Kademlia;
import org.planx.xmlstore.routing.RoutingException;

import core.protocols.p2p.IDHT;
import core.protocols.p2p.Node;
import core.protocols.transport.ITransport;
import core.protocols.transport.socket.request.RequestHandler;
import core.protocols.transport.socket.server.SocketImpl;
import core.tools.HashFunction;
import core.tools.InfoConsole;
import experiments.current.MessageCodes;
import experiments.current.Oracle;



public class KadNode implements IDHT{

	protected String identifier;
	protected ITransport transport;
	protected Node node;
	protected Kademlia kad;
	protected Node manager;
	
	ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);

	/** Hash function */
	protected HashFunction h;
	
	protected KadNode() {}

	public KadNode(String ip, int port, String identifier, String manager, int putDelay) {
		try {
			int port2 = port +1;
			
			this.identifier = identifier;
			this.h = new HashFunction(identifier);
			Identifier kadId = Identifier.randomIdentifier();
			int kadNodeId = Integer.valueOf(kadId.value().toString());
			ServerSocket serverSocket = new ServerSocket(port);
			node = new Node(ip, kadNodeId, serverSocket.getLocalPort());
			//this.manager = new Node(manager);
			kad = new Kademlia(kadId, serverSocket.getLocalPort());
			transport = new SocketImpl(port2, 10, RequestHandler.class.getName(),
					10, 1, 100, this);	
			//put data to its overlay
			
			
			//System.out.println("Create KadNode:"+identifier+","+port+":"+port2+","+kadNodeId);
			((SocketImpl) transport).launchServer();
			String nId = String.valueOf(kadNodeId);
			scheduler.schedule(new InitializeData(nId, nId), putDelay, SECONDS);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void put(String key, String value) {
		//debug
		//System.out.println("kad put: "+key+","+value+" at "+System.currentTimeMillis());
		Identifier id = new Identifier(BigInteger.valueOf(keyToH(key)));
//		System.out.println("put(" + keyToH(key) + ", " + value + ")");
		try {
			kad.put(id, value);
		} catch (RoutingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public String get(String key) {
		int count =0;
		String result = get(key, count);
		return result;
	}

	

	@Override
	public String get(String key, int count) {
		try {
			//	System.out.println("Kad, get StringKey "+key);
			Identifier id = new Identifier(BigInteger.valueOf(keyToH(key)));

			//	System.out.println("Kad, get HKey "+id);	
				String result = (String) kad.get(id);
			//	System.out.println("Kad, get StringKey:"+key+" hkey:"+id+" result "+result);
				return result;
			} catch (RoutingException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} return "err";
	}

	
	
	public void join(String host, int port) {
		try {
			kad.connect(new InetSocketAddress(host, port));
		} catch (RoutingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public String getIdentifier() {
		return identifier;
	}

	public Node getThisNode() {
		return node;
	}

	public ITransport getTransport() {
		return transport;
	}

	public int keyToH(String key) {
		return h.SHA1ToInt(key);
	}

	public void kill() {
		
		
	}

	public String handleRequest(String code) {
//		System.out.println("request handled: " + code);
		String[] args = code.split(",");
		String result = "";
		if (args[0].equals(getIdentifier())) {
			int f = Integer.parseInt(args[1]);
			switch (f) {
			case Oracle.PUT:
				put(args[2], args[3]);
				break;
			case Oracle.GET:
				result = get(args[2]);
				break;
			default:
				break;
			}
		} else if (args[0].equals("getIdentifier")) {
			return getIdentifier();
		}
		else if (args[0].equals(String.valueOf(MessageCodes.DELETE))) {
			result = deleteReceiver();
		}
		else if (args[0].equals(String.valueOf(MessageCodes.MANAGERGET))) {
		//	System.out.println("Kad receive ManagerGET message");
			result = get(args[1]);
		}
		else if (args[0].equals(String.valueOf(MessageCodes.MANAGERCOL))) {
			result = deleteReceiver();
		}
		
				
		return result;
	}
	
	
	public String deleteReceiver()
	{
		//System.out.println("In KadNode: receive DELETE");
		
		String message = "kad,"+kad.getStats(); 
		scheduler.schedule(new StopServer(), 10, SECONDS);
		return message; 

	}
	
	public void setKad(Kademlia kad) {
		this.kad = kad;

	}
	

	public String toString()
	{
	 return kad.toString();	
	}	
	
	
	class StopServer implements Runnable
	{
		public void run()
		{
			
			//System.out.println("KadNode :"+node.toString()+" killed");
			kad.getTransport().close();
			((SocketImpl)transport).stopServer();
			System.exit(0);
		}
	}

	
	class InitializeData implements Runnable
	{
		String key;
		String value;

		public InitializeData(String key, String value)
		{
			this.key = key;
			this.value = value;
		}
		public void run()
		{
		//	System.out.println("Put"+key+":"+value+"to the kad net");
			put(key, value);
		}
	}


	
}
