package experiments.current.synapse.plugin.chord;

import static java.util.concurrent.TimeUnit.SECONDS;

import java.net.InetAddress;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import core.protocols.p2p.Node;
import core.protocols.p2p.chord.IChord;
import core.protocols.transport.socket.server.SocketImpl;
import core.synapse.*;
import experiments.current.MessageCodes;
import experiments.current.node.chord.ChordNode;




/**
 * This is an implementation of a chordPlugin node for a synapse
 * 
 * @author laurent.vanni@sophia.inria.fr - logNet team 2010 - INRIA
 *         Sophia-Antipolis - France
 * Modified by giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France 
 */
public class ChordNodePlugin extends ChordNode {
	ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
	/** the synapse associate with this plugin */
	private Synapse synapse;

	/**
	 * Constructor
	 * 
	 * @param host
	 * @param port
	 * @param synapse
	 * @param overlayIntifier
	 */
	public ChordNodePlugin(String host, int port, Synapse synapse,
			String overlayIntifier) {
		super(host, port, overlayIntifier);
		this.synapse = synapse;
	}

	/**
	 * Constructor
	 * 
	 * @param host
	 * @param port
	 * @param synapse
	 */
	public ChordNodePlugin(String host, int port, Synapse synapse) {
		super(host, port);
		this.synapse = synapse;
	}
	

	public ChordNodePlugin(String host, int port, String overlayIntifier, 
			String manager, int putDelay, Synapse synapse) {
		super(host, port, overlayIntifier, manager, putDelay);
		this.synapse = synapse;
	}


	/**
	 * @see core.protocols.transport.IRequestHandler#handleRequest(String)
	 */
	public String handleRequest(String code) {
		//System.out.println("In ChordPlugin, received code: "+code);
		String[] args = code.split(",");
		String result = "";
	//	System.out.println("In ChordPlugin, args[0]: "+args[0]);
		if (args[0].equals(overlayIntifier)) {
			int f = Integer.parseInt(args[1]);
			switch (f) {
			case IChord.GET:				
			//	System.out.println("In ChordPlugin, received Get request, call SYnapse: "+code);
				result = super.handleRequest(code);
				Node srcNode = new Node(args[4],Integer.parseInt(args[6]));
				new Thread(new SynapseHandleGetRequest(args[2], Integer.parseInt(args[3]), 
						srcNode, synapse)).start();
				break;
			default:
				// IN ALL CASES CONITNUE TO ROUTE
				result = super.handleRequest(code);
				break;
			}
		} else if (args[0].equals(String.valueOf(ISynapse.SYNAPSEOFFER))){
		//	System.out.println("In ChordPlugin, received SYNAPSEOFFER request: "+code);
			Node srcNode = new Node(args[4],Integer.parseInt(args[6]));
			synapse.synapseOfferReceiver(Integer.parseInt(args[1]), Integer.parseInt(args[2]), 
					args[3], srcNode);
		}
		else if (args[0].equals(String.valueOf(MessageCodes.MANAGERCOL))){
			result = deleteReceiver();
			//System.out.println("KadPlugIn receive ManagerCol message");
		}
		else
			result = super.handleRequest(code);
		
		return result;
	}
	class SynapseHandleGetRequest implements Runnable{
	 	   String key;
	 	   Node src;
	 	   Synapse synapse;
	 	   int count;
	 	   public SynapseHandleGetRequest(String key, int count, Node src, Synapse synapse)
	 	   {
	 		   this.key = key;
	 		   this.src = src;
	 		   this.synapse = synapse;
	 		   this.count = count;
	 	   }
	 	   public void run()
	 	   {
	 		   
	 			   synapse.getReceiver(key, count, src);
	 			 // System.out.println("ChordPlugin call Synapse getReceiver");
	 	   }
	    }
	public String deleteReceiver()
	{
		//System.out.println("Chord receive DELETE message");
		endTime = System.currentTimeMillis();
		int duration = Math.round((endTime - startTime)/1000);
		String message = "Pchord,"+messageNumber+","+String.valueOf(duration);
		scheduler.schedule(new StopServer(), 10, SECONDS);
		return message;
		//return messageNumber,duration
	}
		
	class StopServer implements Runnable
	{
		public void run()
		{
			//System.out.println("ChordNodePlugin: "+ getThisNode().toString()+" killed");
			((SocketImpl)transport).stopServer();
	
		}
	}

}
//synapse offer: String message = code +","+ hkey +","+ connectedNetList +"," +getThisNode().toString();