package experiments.current.synapse.plugin.kademlia;

import static java.util.concurrent.TimeUnit.SECONDS;
import java.io.IOException;
import java.math.BigInteger;
import java.net.ServerSocket;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import org.planx.xmlstore.routing.Identifier;
import org.planx.xmlstore.routing.Kademlia;
import org.planx.xmlstore.routing.RoutingException;
import core.protocols.p2p.Node;
import core.synapse.ISynapse;
import core.synapse.Synapse;
import core.protocols.transport.socket.request.RequestHandler;
import core.protocols.transport.socket.server.SocketImpl;
import core.tools.HashFunction;
import core.tools.InfoConsole;
import experiments.current.MessageCodes;
import experiments.current.node.kademlia.KadNode;



public class KadNodePlugin extends KadNode{

	private Synapse synapse;
	protected Node manager;
	ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
	
	public KadNodePlugin(String ip, int port, String identifier, String manager, Synapse synapse, int putDelay) {
		try {
			int port2 = port+1;
			this.identifier = identifier;
			this.synapse = synapse;
			this.h = new HashFunction(identifier);
			ServerSocket serverSocket = new ServerSocket(port);
			
			Identifier kadId = Identifier.randomIdentifier();
			int kadNodeId = Integer.valueOf(kadId.value().toString());
			
			node = new Node(ip, kadNodeId, serverSocket.getLocalPort());
			kad = new Kademlia(kadId, serverSocket.getLocalPort(), this);
			transport = new SocketImpl(port2, 10, RequestHandler.class.getName(),
					10, 1, 100, this);
			
			//put data to its overlay
			
			//System.out.println("Create KadNode:"+identifier+","+port+":"+port2+","+kadNodeId);
			((SocketImpl) transport).launchServer();
			String nId = String.valueOf(kadNodeId);
			scheduler.schedule(new InitializeData(nId, nId), putDelay, SECONDS);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

/*	
	public String get(String key){
		//System.out.println("Kad, get StringKey "+key);
		Identifier id = new Identifier(BigInteger.valueOf(keyToH(key)));
		try {
			//System.out.println("Kad, get HKey "+id);
			String result = (String) kad.get(id);
			System.out.println("Kad, get StringKey:"+key+" hkey:"+id+" result "+result);
			return result;
		} catch (RoutingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} return "err";
	}
*/
	public String get(String key) {
		int count =0;
		String result = get(key, count);
		return result;
	}

	

	@Override
	public String get(String key, int count) {
		try {
			//	System.out.println("Kad, get StringKey "+key);
			Identifier id = new Identifier(BigInteger.valueOf(keyToH(key)));

			//	System.out.println("Kad, get HKey "+id);	
				String result = (String) kad.get(id);
				
				String[] tmp = result.split(",");
			//	System.out.println("Kad, get StringKey:"+key+" hkey:"+id+" result "+result);
				count = count + Integer.parseInt(tmp[1]);
				result= tmp[0]+","+ count;
				return result;
			} catch (RoutingException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} return "err";
	}

	
	public String handleRequest(String code) {

		String[] args = code.split(",");
		String key="";
		String result = "null";
		String srcIp = "";
		int port=0;
		Node srcNode;

		if (args[0].equals(getIdentifier())) { 
			super.handleRequest(code);
	//		System.out.println("nhanh 1");
		} else if (args[0].equals(String.valueOf(ISynapse.SYNAPSEOFFER)))
		{	
			//System.out.println("nhanh 2 synapse offer: "+code);
			srcNode = new Node(args[4],Integer.parseInt(args[6]));
		//	System.out.println("nhanh 2 synapse offer: "+srcNode.toString());
			synapse.synapseOfferReceiver(Integer.parseInt(args[1]), 
					Integer.parseInt(args[2]), args[3], srcNode);		
		}
		else if (args[0].equals(String.valueOf(MessageCodes.MANAGERGET)))
		{
			//System.out.println("KadPlugIn receive ManagerGET message");			
			result = get(args[1]);
		}
		else if (args[0].equals(String.valueOf(MessageCodes.MANAGERCOL))){
			result = deleteReceiver();
			//System.out.println("KadPlugIn receive ManagerCol message");
		}
		else if (code.length()>9){ 
			String substring = code.substring(0,10);
			
			if (substring.equals("DataLookup")){ //receive Get request from overlay
			//		System.out.println("nhanh 3 data lookup request");
			int count=0;	
			for(String arg : args){
				//System.out.println("arg:"+arg);
				if(arg.split("=")[0].equals("lookup")){
					key = arg.split("=")[1].split("]")[0];
				}
				if(arg.split("=")[1].equals("Node[ip")){
						srcIp = arg.split("=")[2];
				}
				if(arg.split("=")[0].equals("port"))
				{	
					port = Integer.parseInt(arg.split("=")[1]);
					//System.out.println("port:"+port);
				}
				if(arg.split("=")[0].equals("count"))
				{	
					count = Integer.parseInt(arg.split("=")[1].split("]")[0]);
					//System.out.println("port:"+port);
				}
			}
		//	System.out.println("KadPlugin call Synapse getReceiver");
			srcNode = new Node(srcIp.toString(), port+1);
			synapse.getReceiver(key, count, srcNode);
			}
		}	
	return result;
	}
	
	public String toString()
	{
	 return kad.toString();	
	}
	
	class InitializeData implements Runnable
	{
		String key;
		String value;

		public InitializeData(String key, String value)
		{
			this.key = key;
			this.value = value;
		}
		public void run()
		{
			//System.out.println("Put"+key+":"+value+"to the kad net");
			put(key, value);
		}
	}
	

	public String deleteReceiver()
	{
		//System.out.println("In KadPlugin: receive DELETE");
		String message = "Pkad,"+kad.getStats(); 
		scheduler.schedule(new StopServer(), 10, SECONDS);
		return message; 

	}
	


	class StopServer implements Runnable
	{
		public void run()
		{
			//System.out.println("KadNodePlugin :"+node.toString()+" killed");
			kad.getTransport().close();
			((SocketImpl)transport).stopServer();
		}
	}

	public void kill() {
	}
}

/*	
 * new Thread(new HandleGetRequest(getIdentifier(), key, src)).start();
	class HandleGetRequest implements Runnable
	{
		String netId;
		String key;
		Node srcNode;
		public HandleGetRequest(String netId, String key, Node srcNode)
		{this.netId = netId;
		this.key = key;
		this.srcNode = srcNode;
		}
		
		public void run()
		{
			synapse.getReceiver(netId,key, srcNode);
		}
	}
	*/
//plugin forward the Synapse offer to Synapse
//node: return "Node[ip="+ip.getHostAddress()+",port="+port+",id="+id+"]";