/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */

package manager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.ConcurrentHashMap;

import messages.MessageCodes;

public class AliveNodes {
	//netId --> arrayList<node>
	public ConcurrentHashMap<String, ArrayList<NodeInfo>> nodes;
	Random randomGenerator;
	
	public AliveNodes(){
		nodes = new ConcurrentHashMap<String, ArrayList<NodeInfo>>();
		randomGenerator = new Random();
	}
	
	
	public String getBootstrapNode(String netID)
	{
		//debug
		//printNodes();
		
		
		String result = "null";
		if (nodes.containsKey(netID))
		{
			int n = nodes.get(netID).size();
			if (n>0)
			{	
				Random randomGenerator = new Random();
				int index = randomGenerator.nextInt(n);
				result = nodes.get(netID).get(index).getNode().toString();
			}
		}
//		System.out.println("Bootstrap node for "+netID+" is "+result);
		return result;
	}


	public void putNode(String netID, NodeInfo node) {
//		System.out.println("received the ADDNODE: "+netID+" : "+node);

		if ((nodes.get(netID)) != null) {
			nodes.get(netID).add(node);
		} else {

			ArrayList tmp = new ArrayList<NodeInfo>();
			tmp.add(node);		
			ArrayList result = nodes.putIfAbsent(netID, tmp);
			if (result != null)
				nodes.get(netID).add(node);
			
		}	
	}
	public void removeNode(String netId, NodeInfo nodeInfo) {		
		//System.out.println("AliveNodes, Receive remove request: "+netId+":"+nodeInfo.toString());
		
		//System.out.println("AliveNodes "+nodes.get(netId).toString());
		
		if (nodes.get(netId).contains(nodeInfo))
		{
			nodes.get(netId).remove(nodeInfo);
		//	System.out.println("AliveNodes, remove: "+netId+":"+nodeInfo.toString());
		}
	}
	
	// get a synapse which will lookup the key
	public Node getSynapse()
	{
		ArrayList<NodeInfo> synNodes = new ArrayList<NodeInfo>();
		Node result = null; 
		if (nodes.size() > 0)
		{
			synNodes = nodes.get("synapse");
			int size = synNodes.size();
			if (size>0)
			{	
				int index = randomGenerator.nextInt(size);
				result = synNodes.get(index).getNode();
			}
		}
		//debug
		//System.out.println("Synapse looking up: "+result.toString());
		return result;
	}
	
	// get a ChordNode which will lookup the key
	public String getDataNode()
	{
		Node node = null;
		String nodeType="";
		int size = nodes.size();				
		if (size > 0)
		{
			String netId = "";	
			while (true)
			{	
				int netIndex = randomGenerator.nextInt(size);
				Iterator iter = nodes.keySet().iterator();
				for (int i=0; i<= netIndex;i++)
					netId = (String)iter.next(); //get netId
				if (!netId.substring(0,3).equals("syn"))// && nodes.get(netId).size()>0)
					break;
			}
			int nodeSize = nodes.get(netId).size();
			//System.out.println("GET net sizes "+nodeSize);
			if (nodeSize>0)
			{
				
				int nodeIndex = randomGenerator.nextInt(nodeSize);	
				NodeInfo nodeInfo =  nodes.get(netId).get(nodeIndex);
				
				if (netId.substring(0,3).equals("kad"))
				{	
					nodeType="kad";
					node = (new Node(nodeInfo.getNode().getIp(),nodeInfo.getPort()));
				}	
				else
				{	
					nodeType="chord";
					node = nodeInfo.getNode();
				}	
			}
		}
		return nodeType+":"+node.toString();
	}

			

	public String getLocalNodeAndKey()
	{
		String result ="null";
		int size = nodes.size();
				
		if (size > 0)
		{
			String netId = "";
			while (true)
			{	
				int netIndex = randomGenerator.nextInt(size);
				Iterator iter = nodes.keySet().iterator();
				for (int i=0; i<= netIndex;i++)
					netId = (String)iter.next(); //get netId
				if (!netId.substring(0,3).equals("syn"))// && nodes.get(netId).size()>0)
					break;
			}
			
			//debug
			//System.out.println("GET netId: "+netId);
			
			int nodeSize = nodes.get(netId).size();
			//System.out.println("GET net sizes "+nodeSize);
			if (nodeSize>0)
			{
				String node;
				int keyIndex = randomGenerator.nextInt(nodeSize);
				String key = String.valueOf(nodes.get(netId).get(keyIndex)
						.getNode().getId());
				
				int nodeIndex = randomGenerator.nextInt(nodeSize);	
				
				if (netId.substring(0,3).equals("kad"))
					node = (new Node(nodes.get(netId).get(nodeIndex).getNode().getIp(),
							nodes.get(netId).get(nodeIndex).getPort())).toString();
				else
					
					node = nodes.get(netId).get(nodeIndex).getNode().toString();
				
				
				result = netId+":"+key+":"+node;
			}
		}
		return result;
	}

	
	//get a key to lookup, key = id of a live node
	public String getKey()
	{
		String result ="null";
		int size = nodes.size();
		
		//debug
		//System.out.println("In GET nodes size: "+size);
		
		if (size > 0)
		{
			String netId = "";
			while (true)
			{	
				int netIndex = randomGenerator.nextInt(size);
				Iterator iter = nodes.keySet().iterator();
				for (int i=0; i<= netIndex;i++)
					netId = (String)iter.next(); //get netId
				if (!netId.substring(0,3).equals("syn"))
					break;
			}
			
			//debug
		//	System.out.println("GET netId: "+netId);
			
			int nodeSize = nodes.get(netId).size();
		//	System.out.println("GET net sizes "+nodeSize);
			if (nodeSize>0)
			{
				int nodeIndex = randomGenerator.nextInt(nodeSize);		
				result = String.valueOf(nodes.get(netId).get(nodeIndex).getNode().getId());
			}
		}
		return result;
	}

	
	
	
	
	
	public ConcurrentHashMap<String, ArrayList<NodeInfo>> getNodes()
	{
		return nodes;
	}
	
	public ArrayList<Node> getAll()
	{
		ArrayList<Node> result = new ArrayList<Node>();
//		/System.out.println(" ColStats: number of elements: "+ nodes.keySet().size());
		for (Map.Entry<String , ArrayList<NodeInfo>> entry : nodes.entrySet())
		{
			
			NodeInfo nodeInfo;
			Iterator iter = entry.getValue().iterator();
			String netId = entry.getKey();
			//System.out.println("ColStats, netId: "+netId+" delete number of nodes :"+entry.getValue().size());
			if (netId.substring(0,3).equals("kad"))
			{	
				
				while (iter.hasNext())
				{
					nodeInfo = (NodeInfo)iter.next();
					Node dest = new Node(nodeInfo.getNode().getIp(), nodeInfo.getPort()); 
					result.add(dest);
				}
			}
			else 
			{
				while (iter.hasNext())
				{
					
					nodeInfo = (NodeInfo)iter.next();
					Node dest = nodeInfo.getNode(); 
					result.add(dest);					
				}
			}	
		}
		return result;

	}
	public void printNodes()
	{
		System.out.println("PrintNodes:");
		for (Map.Entry<String , ArrayList<NodeInfo>> entry : nodes.entrySet())
		{
			
			NodeInfo nodeInfo;
			System.out.println("Key: "+entry.getKey());
			Iterator iter = entry.getValue().iterator();
			while (iter.hasNext())
			{
				
				nodeInfo = (NodeInfo)iter.next();
				System.out.println("Node: "+nodeInfo.toString());
			}
			
		}
	}
	
	
}
