/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */

package manager;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import static java.util.concurrent.TimeUnit.SECONDS;
import manager.NodeInfo;
import tools.InfoConsole;
import transport.IRequestHandler;
import transport.socket.request.*;
import transport.socket.server.*;
import transport.ITransport;
import messages.*;
import operations.CreateOverlay;
import operations.Lookup;
import operations.CollectStatistics;
import tools.CheckFinish;
import tools.InitStatisticFiles;


public class Manager implements  IRequestHandler{
	
	protected AliveNodes aliveNodes; //alive nodes on all overlays  
	protected Configuration config;	 //config info	
	protected Machines machines;	
	protected ITransport transport;
	protected Node manager;			
	protected Lookup lookup;
	protected CollectStatistics collectStatistics;
	protected static final int managerPort = 5000;
	protected TimeCheck timeCheck; //for debugging only
	protected CheckFinish checkFinish;
	protected InitStatisticFiles initStatisticFiles; 
	protected ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
		
	
public Manager()
{	
	String confFile = "experiment.conf";
	String machinesList = "machines.conf";
	
	//System.out.println("confFile"+confFile);
	//System.out.println("machinesList"+machinesList);
	
	checkFinish = new CheckFinish();
	initStatisticFiles = new InitStatisticFiles();
	aliveNodes = new AliveNodes();
	config = new Configuration(confFile);
	machines = new Machines(machinesList, false);
	manager = new Node(machines.managerIp, managerPort);
	System.out.println("manager: "+manager.toString());
	transport = new SocketImpl(managerPort, 10, RequestHandler.class.getName(),10, 1, 100, this);			
	((SocketImpl) transport).launchServer();
}


public void launch()
{		
	config.setStartTime();
	config.toString();
	//debug 
	timeCheck = new TimeCheck(config);
	initStatisticFiles.init();
	lookup = new Lookup(config, aliveNodes, transport, timeCheck);
	collectStatistics = new CollectStatistics(config, aliveNodes, transport, timeCheck, checkFinish);

	createOverlays();
	collectStatistics.start();
	lookup.start();		
	
	long stopTime = (long)(config.simulationTime + 300);
	scheduler.schedule(new Stop(), stopTime, SECONDS);
}	



public void createOverlays()
{
	if ((config.numberOfChordNets == 0) && (config.percentageOfSynnapse == 0))
	{
		(new Thread(new CreateOverlay("KAD","kad0", config, aliveNodes, 
				transport, machines, manager, timeCheck))).start();
		
		try {
			Thread.sleep(50000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		for (int i=0; i< 19; i++)
			(new Thread(new CreateOverlay("KAD","kad0", config, aliveNodes, 
					transport, machines, manager, timeCheck))).start();

	}
	else if ((config.numberOfKadNets == 0) && (config.percentageOfSynnapse == 0))
	{
		(new Thread(new CreateOverlay("CHORD","chord0", config, aliveNodes, 
				transport, machines, manager, timeCheck))).start();

		try {
			Thread.sleep(50000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		for (int i=0; i< 19; i++)
			(new Thread(new CreateOverlay("CHORD","chord0", config, aliveNodes, 
					transport, machines, manager, timeCheck))).start();

		
		
	}
	else
	{
			
		for (int i=0; i< config.numberOfChordNets; i++)
		{
			(new Thread(new CreateOverlay("CHORD","chord"+i, config, aliveNodes, 
					transport, machines, manager, timeCheck))).start();
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}	

		for (int i=0; i< config.numberOfKadNets; i++)
		{
			(new Thread(new CreateOverlay("KAD","kad"+i, config, aliveNodes, 
					transport, machines, manager, timeCheck))).start();
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}	

		if (config.percentageOfSynnapse != 0)
		{	
			for (int i=0; i< (config.numberOfChordNets + config.numberOfKadNets)/2; i++)	
			{
				(new Thread(new CreateOverlay("SYNAPSE", "synapse", config, aliveNodes, transport, 
				machines, manager, timeCheck))).start();
				try {
					Thread.sleep(2000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}	

		}
	}
}

public String handleRequest(String message)
{
	//System.out.println("manager receive message :"+message);
	String[] args = message.split(",");
	String result = "";
	String netId;
	
		int f = Integer.parseInt(args[0]);
		switch (f) {
		case MessageCodes.GETCONNECTION:
				
				netId = args[1];
				
				result = aliveNodes.getBootstrapNode(netId).toString();
			break;
		case MessageCodes.ADDNODE:
			
				netId = args[1];
				Node node = new Node(args[2], Integer.parseInt(args[3]),
						Integer.parseInt(args[4]));
				NodeInfo newNode = new NodeInfo(node, Integer.parseInt(args[5]));
			//	System.out.println("addnode required"+newNode.toString());
				aliveNodes.putNode(netId, newNode);		
			break;					
		}
	
	return result;
}

public void kill() {
	((SocketImpl) transport).stopServer();
}

public String toString()
{
	return manager.getIp()+ ","+ 0 +","+manager.getPort();
	
}

class Stop implements Runnable{
	public void run()
	{
		/*while (checkFinish.getFinish() == false)
		{
			System.out.println("Manger , finish = false");
			try {
				Thread.currentThread().sleep(5000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}*/	
		timeCheck.check("Stop the application");
		((SocketImpl) transport).stopServer();
		System.exit(0);
	}
}
}
