/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */

package operations;

import static java.util.concurrent.TimeUnit.SECONDS;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import manager.AliveNodes;
import manager.Configuration;
import manager.Machines;
import manager.TimeCheck;
import messages.ColStatsMessage;
import messages.MessageCodes;
import manager.NodeInfo;
import tools.WriteToFile;
import transport.ITransport;
import manager.Node;
import tools.CheckFinish;


public class CollectStatistics{
protected Random randomGenerator;
protected AliveNodes aliveNodes;   
protected Configuration config;	 	
protected Machines machines;	
protected ITransport transport;
protected WriteToFile writeToFile;
protected TimeCheck timeCheck;
CheckFinish checkFinish;

protected ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);


	public CollectStatistics(Configuration config, AliveNodes aliveNodes, 
			ITransport transport, TimeCheck timeCheck, CheckFinish checkFinish)
	{
		this.aliveNodes = aliveNodes;
		this.transport = transport;
		this.config = config;
		writeToFile = new WriteToFile();
		randomGenerator = new Random();
		this.timeCheck = timeCheck;
		this.checkFinish = checkFinish;
	}
	
	public void start()
	{
		//debug
	//	System.out.println("Schedule CollectStats at "+config.simulationTime);
		scheduler.schedule(new ColStats(), (long)(config.simulationTime +60), SECONDS);
	}
	
	

	class ColStats implements Runnable
	{
	
		public void run()
		{
			//System.out.println("Run ColStats: number of elements: "+aliveNodes.getNodes().keySet().size());
			
			String message = (new ColStatsMessage(MessageCodes.MANAGERCOL)).toString();
			ArrayList<Node> dests = aliveNodes.getAll(); 
			//System.out.println("All Node"+dests.toString());
			for (int i=0; i<dests.size(); i++)
			{
				Node dest = dests.get(i);
				String event = "send collect statistic to : "+dest.toString();
				//timeCheck.check(event);
				String res = transport.sendRequest(message, dest);
				//System.out.println("res = "+ res);
				if (!res.equals("null") && !res.equals("unreachable")) 
				{
					 String filename = "MessageNumbers.txt";
					 writeToFile.write(filename, res);											
				}
			}
			
			
	/*		for (Map.Entry<String , ArrayList<NodeInfo>> entry : aliveNodes.getNodes().entrySet())
			{
				
				NodeInfo nodeInfo;
				Iterator iter = entry.getValue().iterator();
				String netId = entry.getKey();
				System.out.println("In netId: "+netId+" delete number of nodes :"+entry.getValue().size());
				if (netId.substring(0,3).equals("kad"))
				{	
					
					while (iter.hasNext())
					{
						nodeInfo = (NodeInfo)iter.next();
						Node dest = new Node(nodeInfo.getNode().getIp(), nodeInfo.getPort()); 
						String message = String.valueOf(MessageCodes.MANAGERCOL);
						
						//debug
						String event = "send collect statistic to : "+dest.toString();
						timeCheck.check(event);
						
						
						String res = transport.sendRequest(message, dest);
						System.out.println("res = "+ res);
						if (!res.equals("null") && !res.equals("unreachable")) 
						{
							 String filename = "MessageNumbers.txt";
							 writeToFile.write(filename, res);											
						}
						aliveNodes.removeNode(netId, nodeInfo);
					}
				}
				else 
				{
					while (iter.hasNext())
					{
						
						nodeInfo = (NodeInfo)iter.next();
						Node dest = nodeInfo.getNode(); 
						String message = String.valueOf(MessageCodes.MANAGERCOL);
						
						
						String event = "send collect statistic to : "+dest.toString();
						timeCheck.check(event);
						
						String res = transport.sendRequest(message, dest);
						
						System.out.println("res = "+ res);
						
						if (!res.equals("null") && !res.equals("unreachable")) 
						{
							 String filename = "MessageNumbers.txt";
							 writeToFile.write(filename, res);							
						}
						aliveNodes.removeNode(netId, nodeInfo);
					}
				}	
			}
	*/	
		//checkFinish.setFinish();
		//System.out.println("Colstats , finish = true");
		}
	}	
}