/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */

package operations;

import static java.util.concurrent.TimeUnit.SECONDS;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.ScheduledExecutorService;
import tools.CommandExecutor;
import tools.ParetoDistribution;
import transport.ITransport;
import manager.Configuration;
import manager.Machines;
import manager.Node;
import manager.NodeInfo;
import manager.AliveNodes;
import manager.TimeCheck;

class CreateNode implements Runnable
{
	
	protected double lifetime;
	protected double meanLifetime;
	protected double meanDeadtime;
	protected Machines machines;
	protected AliveNodes aliveNodes;
	protected String netId;
	protected String nodeType;
	protected Node manager;
	protected Configuration config;
	protected CommandExecutor commandExecutor;
	protected Random randomGenerator;
	protected ITransport transport;
	ScheduledExecutorService scheduler;
	protected TimeCheck timeCheck;
	
	
	
	 public CreateNode(ScheduledExecutorService scheduler, double lifetime, 
			 double meanLifetime, double meanDeadtime, Machines machines, String nodeType,
			 String netId, Node manager, Configuration config, AliveNodes aliveNodes, 
			 ITransport transport, TimeCheck timeCheck)
	 {			
		 this.scheduler = scheduler;	 
		 this.lifetime = lifetime;
		 this.meanDeadtime = meanDeadtime;
		 this.meanLifetime = meanLifetime;			 			 
		 this.machines = machines;
		 this.nodeType = nodeType;
		 this.netId = netId;
		 this.manager = manager;
		 this.config = config;
		 this.transport = transport;
		 randomGenerator = new Random();
		 commandExecutor = new CommandExecutor();
		 this.aliveNodes = aliveNodes;
		 this.timeCheck = timeCheck;
	 }

	 
	 public void run()	 
	 {
		 //debug

			double currentTime = (System.currentTimeMillis()-config.startTime)/1000;
			//System.out.println("Create nodetype "+nodeType);
			if (currentTime  > config.simulationTime)
				return;
		 
		 CreateOverlay.nodeCounter++;
		 System.out.println("Count = "+CreateOverlay.nodeCounter);
		 
	 	 if (nodeType.equals("SYNAPSE"))	 	 
	 		createSynapse();	
	 	 else 
	 		createDataNode();
	 }	 
		 
		 	 
	public void createDataNode()
	{
						
		
		String machine;
		String machineIp;
		int port;
		int port2;
		
		
		
		if (nodeType.equals("CHORD"))
		{	
			
			machine = machines.getMachine("CHORD", 0); 
			
			//System.out.println("create node: Chord, machine "+machine.toString());
			
			machineIp = machine.split(",")[0];
			port = Integer.valueOf(machine.split(",")[1]);
			port2 = port;
			
		}
		else 
		{	
			
			machine = machines.getMachine("KAD", 0); 
			//System.out.println("create node: Kad machine "+ machine.toString());
			machineIp = machine.split(",")[0];
			port = Integer.valueOf(machine.split(",")[1]);
			port2 = port+1;
			
		}
		
		int putDelay = getPutDelay();

		String bootstrapNode = aliveNodes.getBootstrapNode(netId);
		
					
		String command = "oarsh "+ machineIp+" java -jar JSynapse2.0.jar " 
			+"node "+ nodeType+" "+ machineIp+" "+ port + " "+ netId +" "+ bootstrapNode 
			+" "+ manager + " "+ putDelay;
		
	
		
	/*	
		String command = "java -jar JSynapse2.0.jar " +"node "+ 
		nodeType+" "+ machineIp+" "+ port + " "+ netId +" "+ bootstrapNode 
			+" "+ manager + " "+ putDelay;		
	*/	
		System.out.println("Create node: "+command);
		
		//debug
		//timeCheck.check(command);
		
		
		commandExecutor.execute(command);

		if (config.churn == true)
		{	
			Node node = new Node(machineIp, port);
			NodeInfo nodeInfo = new NodeInfo(node, port2);
		
		
			//schedule to delete this Synapse node and its plugin
			double currentTime = (System.currentTimeMillis()-config.startTime)/1000; 
			
			//schedule to delete created Node if the delete time < simulation time
			if (currentTime + lifetime < config.simulationTime)
				scheduler.schedule(new DeleteNode(scheduler, null, nodeInfo, meanLifetime, 
						meanDeadtime, nodeType, config, transport, aliveNodes, machines, netId, manager, timeCheck)
						, (long)lifetime, SECONDS);

		}

	 }
	 
	 public void createSynapse()
	 {
		ArrayList<CreatedNodeInfo> list = new ArrayList<CreatedNodeInfo>();
		
		int netNumbers = config.synDegree;
		/*
		int n = config.numberOfChordNets + config.numberOfKadNets; //set of netID
		//Synapse connected to maximum 4 networks
		if (n < 4)
			netNumbers = randomGenerator.nextInt(n) + 1;
		else 
			netNumbers = randomGenerator.nextInt(4) + 1;
		
		//debug 
		 */
	//	System.out.println("Synapse connected to " +netNumbers + " overlays");
		
		String machine = machines.getMachine("SYNAPSE", netNumbers); 
		
		String machineIp = machine.split(",")[0];
		
		int port = Integer.valueOf(machine.split(",")[1]);	
		
		String command = "oarsh "+ machineIp+" java -jar JSynapse2.0.jar " 
			+"synapse" + " "+ machineIp+" "+port+" "+manager.toString(); 

		
	//	String command = "java -jar JSynapse2.0.jar " +"synapse" 
		//		+ " "+machineIp+" "+port+" "+manager.toString();
		
		
		if (config.churn == true)
		{	
	
			Node node = new Node(machineIp, port);
			NodeInfo nodeInfo = new NodeInfo(node, port);	
			CreatedNodeInfo createdNodeInfo = new CreatedNodeInfo("synapse", nodeInfo);
			list.add(createdNodeInfo);
		}	
		
		
		
		command = command +" "+ getPluginNodes(list, netNumbers, port+1, machineIp);
		
		//debug
		//timeCheck.check(command);
		
		
		System.out.println("Create node: "+command);
		
		commandExecutor.execute(command);	
	
		if (config.churn == true)
		{	
			//schedule to delete this Synapse node and its plugin
			double currentTime = (System.currentTimeMillis()-config.startTime)/1000; 
			
			//schedule to delete created Node if the delete time < simulation time
			if (currentTime + lifetime < config.simulationTime)
				scheduler.schedule(new DeleteNode(scheduler, list, null, meanLifetime,meanDeadtime, 
						nodeType, config, transport, aliveNodes, machines, netId, manager, timeCheck)
						, (long)lifetime, SECONDS);
		}	
	 }
	 
	 public String getPluginNodes(ArrayList<CreatedNodeInfo> list, int netNumbers, 
			 int port, String machineIp)
	 {


		String command = "";
		ArrayList<String> usedNetId = new ArrayList<String>();
		int putDelay = getPutDelay();
		
		for (int i=1; i<= netNumbers;i++)
		{
			CreateOverlay.nodeCounter++;
			System.out.println("Count = "+CreateOverlay.nodeCounter);
				//store used netId
			String id;
			while(true)
			{
				int type; 
				int index;
				
				if ((config.numberOfChordNets != 0) && (config.numberOfKadNets == 0)) 
					type = 0;
				else if ((config.numberOfChordNets == 0) && (config.numberOfKadNets != 0)) 
					type = 1;
				else 
					type = randomGenerator.nextInt(2);
				
				if (type == 0){ //chord	
					index = randomGenerator.nextInt(config.numberOfChordNets);
					id = "chord"+index;
				}
				else {
					index = randomGenerator.nextInt(config.numberOfKadNets);
					id = "kad"+index;				
				}
				synchronized(usedNetId)
				{
					if (!usedNetId.contains(id)) 
					{
						//System.out.println("Synapse connected to "+id);
						usedNetId.add(id);
						break;
					}
				}
			}
			//get bootstrapNode for the plugin Node			
			String bootstrapNode = aliveNodes.getBootstrapNode(id);
			
			if (id.substring(0,3).equals("kad"))
			{	
				command = command + " -a kad "+ port+ " "+ 
					id +" "+ bootstrapNode + " "+ putDelay;

				Node node = new Node(machineIp, port);
				NodeInfo nodeInfo = new NodeInfo(node, port+1);	
				CreatedNodeInfo createdNodeInfo = new CreatedNodeInfo(id, nodeInfo);
				list.add(createdNodeInfo);
				port = port + 2;
			}
			else 
			{	
				command = command + " -a chord "+ port+ " "+ id +" "
				+ bootstrapNode + " "+ putDelay;
				Node node = new Node(machineIp, port);
				NodeInfo nodeInfo = new NodeInfo(node, port);		
				
				CreatedNodeInfo createdNodeInfo = new CreatedNodeInfo(id, nodeInfo);
				list.add(createdNodeInfo);
				port = port + 1;

			}

		}
		return command;
		 
	 }
		 
	 public int getPutDelay()
	 {
		 int putDelay = 2;
		 /*// for any data
		 double currentTime = (System.currentTimeMillis()-config.startTime)/1000; 
		 int putDelay = 30; //the node call PUT method after init at least 10 second 
		 
		 if ( currentTime  < config.initTime+config.transitionTime)
		 putDelay = Math.abs((int)(config.initTime+config.transitionTime - currentTime 
				 + randomGenerator.nextInt((int)config.putTime-30)));
		*/
		 
		 return putDelay;
	 }
}

