/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */

package operations;

import static java.util.concurrent.TimeUnit.SECONDS;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.concurrent.ScheduledExecutorService;
import messages.DeleteMessage;
import messages.MessageCodes;
import manager.AliveNodes;
import manager.Configuration;
import manager.Machines;
import manager.Node;
import manager.NodeInfo;
import manager.TimeCheck;
import operations.CreatedNodeInfo;
import tools.ParetoDistribution;
import tools.WriteToFile;
import transport.ITransport;

class DeleteNode implements Runnable
{
    protected double meanLifetime;
    protected double meanDeadtime;
    protected NodeInfo nodeInfo;
    protected Configuration config; 
    protected ITransport transport;
    protected AliveNodes aliveNodes;
    protected Machines machines;
    protected Node manager;
    protected String netId;
    protected ArrayList<CreatedNodeInfo> list = null;
    protected String nodeType;
    protected Node dest = null;
    protected ScheduledExecutorService scheduler;
    protected ParetoDistribution pareto;
    protected WriteToFile writeToFile;
    protected TimeCheck timeCheck;
    
    public DeleteNode(ScheduledExecutorService scheduler, ArrayList<CreatedNodeInfo> list, 
    		NodeInfo nodeInfo, double meanLifetime, double meanDeadtime, String nodeType,
    		Configuration config, ITransport transport, AliveNodes aliveNodes, Machines machines, 
    		String netId, Node manager, TimeCheck timeCheck)
	 {
		 this.scheduler = scheduler;
		 this.meanLifetime = meanLifetime;
		 this.meanDeadtime = meanDeadtime;
		 this.config =config;
		 this.netId = netId;
		 this.manager = manager;
		 this.nodeType = nodeType;
		 if (list == null) 
			 this.nodeInfo = nodeInfo;
		 else 
			 this.list = list;
		 this.transport = transport;
		 this.aliveNodes = aliveNodes;
		 pareto = new ParetoDistribution();
		 writeToFile = new WriteToFile();
		 this.timeCheck = timeCheck;
	 }

		 
		 public void run()
		 {			  
			//debug
				double currentTime = (System.currentTimeMillis()-config.startTime)/1000;
				System.out.println("Delete Node, current time:"+currentTime);
				if (currentTime  > config.simulationTime)
					return;

	
			 CreateOverlay.nodeCounter--;
			 System.out.println("Count = "+CreateOverlay.nodeCounter);
			 
			 //remove node from nodes table
				//in case of Synapse -> remove its plugins 		
				if (nodeType.equals("SYNAPSE"))
				{
					int size = list.size();
					for (int i=0; i< size; i++)	
					{
						String netId = list.get(i).netId;
						NodeInfo nodeInfo = list.get(i).nodeInfo;
						aliveNodes.removeNode(netId, nodeInfo);	
						//System.out.println("removeSynapse and Plugins:"+netId+":"+nodeInfo.toString());
					}
				}
				else {
					aliveNodes.removeNode(netId, nodeInfo);
					//System.out.println("removeData Node:"+netId+":"+nodeInfo.toString());
				}

			 
			 
			 DeleteMessage deleteMessage = new DeleteMessage(String.valueOf(MessageCodes.DELETE));
			  
			 if (nodeType.equals("KAD"))
				dest = new Node(nodeInfo.getNode().getIp(), nodeInfo.getPort()); 
			else if  (nodeType.equals("CHORD"))
				dest = nodeInfo.getNode();
			else //synapse 
				dest = list.get(0).nodeInfo.getNode();
			 
	
			 //debug
			 double tmpTime = (Math.round(System.currentTimeMillis()-config.startTime)/1000);
			// System.out.println("Send Delete message "+deleteMessage.toString()+" to Node "+ 
			//	dest.toString()+ " at "+tmpTime);	 
			 
			//debug
			String event = "send delete message to "+dest.toString();
		//	timeCheck.check(event);
			 
			 String res = transport.sendRequest(deleteMessage.toString(), dest);

			 
			 if (!res.equals("null") && !res.equals("unreachable"))
			 {
				 String filename = "MessageNumbers.txt";
				 writeToFile.write(filename, res);
			 }
			//message: synapse:messageNumber:duration, [chord:messageNumber:duration,
			// kad:messageNumber:duration]

			 

			 //	Manager.addTraffic(Integer.parseInt(res)); 
			//debug
	 
			 
			 if (nodeType.equals("SYNAPSE")) netId ="synapse";

			 currentTime = (System.currentTimeMillis()-config.startTime)/1000;
			 
			long scheduledCreateTime = (long)pareto.individualLifetime(meanDeadtime); 
	
			if (currentTime+ scheduledCreateTime <  config.simulationTime)
			 scheduler.schedule(new CreateNode(scheduler, pareto.individualLifetime(meanLifetime), 
			meanLifetime, meanDeadtime, machines, nodeType, netId, manager, config, aliveNodes, 
			 transport, timeCheck), scheduledCreateTime, SECONDS);	 
		 }
	
	}


