/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */

package operations;

import static java.util.concurrent.TimeUnit.SECONDS;
import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import manager.AliveNodes;
import messages.ManagerGetMessage;
import messages.MessageCodes;
import manager.Configuration;
import manager.Machines;
import manager.Node;
import manager.TimeCheck;
import tools.WriteToFile;
import transport.ITransport;


public class Lookup {
	protected AliveNodes aliveNodes;   
	protected Configuration config;	 	
	protected Machines machines;	
	protected ITransport transport;
	protected ScheduledExecutorService scheduler1 = Executors.newScheduledThreadPool(1);
	protected ScheduledExecutorService scheduler2 = Executors.newScheduledThreadPool(1);
	//protected ScheduledExecutorService scheduler3 = Executors.newScheduledThreadPool(1);
	protected Random randomGenerator;
	protected static int dataCount = 0;
	protected static int synCount = 0;
	protected static int synSuccess = 0;
	protected static int synFailed = 0;
	protected WriteToFile writeToFile;
	protected int getNums;
	protected TimeCheck timeCheck;
	
	public Lookup(Configuration config, AliveNodes aliveNodes, ITransport transport, TimeCheck timeCheck)
	{
		this.aliveNodes = aliveNodes;
		this.transport = transport;
		this.config = config;
		writeToFile = new WriteToFile();
		randomGenerator = new Random();
		this.timeCheck = timeCheck;
	}
	
	public void start()
	{
		//in put/get phase 
		getNums = (int)Math.round(config.measurementTime/config.getInterval);
		
		long delay = (long)(config.initTime + config.transitionTime+config.putTime);
		//debug
		//System.out.println("Schedule Get at "+delay);
		//schedule the first get
		
		if ( config.percentageOfSynnapse != 0 )
			scheduler1.schedule(new SynapseLookup(), delay, SECONDS);
		else 
			scheduler2.schedule(new LocalOverlayLookup(), (delay+3), SECONDS);
		//scheduler3.schedule(new DataNodeLookup(), (delay+6), SECONDS);
	}
	//time: init+ transition + put + get/put

	
	class SynapseLookup implements Runnable
	{
	
		public void run()
		{
			
			Node dest = aliveNodes.getSynapse();
			String key = aliveNodes.getKey();
			if (!(dest == null) && !(key.equals("null")))
			{	
				String managerGetMessage;			
				managerGetMessage = new ManagerGetMessage(MessageCodes.MANAGERGET, key).toString();
				
				//debug
				//String event = "lookup for Key: " + key +" dest "+dest.toString();
				//timeCheck.check(event);
				
				String result = transport.sendRequest(managerGetMessage, dest);
				
				//debug
				//System.out.println("In Synapse Lookup --> result: "+result);
				/*	if (result.equals("key"))
						synSuccess++;
					else 
						synFailed++;
				 */	
					result = key+","+result;
					 String filename = "SynapseLookup.txt";
					 writeToFile.write(filename, result);					
			}	 
				 		  
			double currentTime = (System.currentTimeMillis()-config.startTime)/1000;
			 synCount++;
			 if ((synCount < getNums) && (currentTime < config.simulationTime)) 			
				 scheduler1.schedule(new SynapseLookup(), (long)config.getInterval, SECONDS);
		}
	}

//
	class DataNodeLookup implements Runnable
	{
	
		public void run()
		{
			String code = aliveNodes.getDataNode();
			String nodeType = code.split(":")[0];
			Node dest = new Node(code.split(":")[1]);
			String key = aliveNodes.getKey();
			
			if (!(dest == null) && !(key.equals("null")))
			{	
				String managerGetMessage;			
				managerGetMessage = new ManagerGetMessage(MessageCodes.MANAGERGET, key).toString();
				
				//debug
				//String event = "nodeType"+nodeType+ "DataNode lookup for Key: " + key +" dest "+dest.toString();
				//timeCheck.check(event);
				
				String result = transport.sendRequest(managerGetMessage, dest);
				
				//debug
//			/	System.out.println("In DataNodeLookup --> result: "+result);
				/*	if (result.equals("key"))
						synSuccess++;
					else 
						synFailed++;
				 */	
					result = key+","+result;
					 String filename = nodeType+"_DataNodeLookup.txt";
					 writeToFile.write(filename, result);					
			}	 
				 		  
			double currentTime = (System.currentTimeMillis()-config.startTime)/1000;
			 synCount++;
			 if ((synCount < getNums) && (currentTime < config.simulationTime)) 			
				 scheduler1.schedule(new DataNodeLookup(), (long)config.getInterval, SECONDS);
		}
	}

	
	
	class LocalOverlayLookup implements Runnable
	{
	
		public void run()
		{
			//System.out.println("Start data node Lookup");
			String result = aliveNodes.getLocalNodeAndKey();
			if (!result.equals("null"))
			{	
				String netId = result.split(":")[0];
				String key = result.split(":")[1];
				Node dest = new Node(result.split(":")[2]);
				
			if (!(dest == null) && !(key.equals("null")))
			{	
				String managerGetMessage;			
				managerGetMessage = new ManagerGetMessage(MessageCodes.MANAGERGET, key).toString();
				
				//debug
				//String event = "LocalOverlay lookup for Key: " + key+" netId:"+netId +" dest "+dest.toString();
				//timeCheck.check(event);
				
				String res = transport.sendRequest(managerGetMessage, dest);
				
				//debug
			//	System.out.println("In Data Lookup --> result: "+res);
					res = key+","+res;
					String net = netId.substring(0,3);
					 String filename = net+"_LocalOverlayLookup.txt";					
					 writeToFile.write(filename, res);					
			}	 
				 		  
			double currentTime = (System.currentTimeMillis()-config.startTime)/1000;
			 dataCount++;
			 if ((dataCount < getNums) && (currentTime < config.simulationTime)) 			
				 scheduler2.schedule(new LocalOverlayLookup(), (long)config.getInterval, SECONDS);
			}
		}		
	}


}

