package tools;

public class ParetoDistribution {
	protected static final double PI = 3.14159265;
	
	public double individualMeanTime(double mean)
	{
	    return shiftedPareto(3, betaByMean(mean));
	}

	public double betaByMean(double mean)
	{
		double alpha = 3;
		return 1/(mean*(alpha -1));
	}
	
	public double shiftedPareto(double a, double b)
	{
		int rng = 0;
	    return (pareto_shifted(a, b, 0, rng)/b - 1) / b;
	}
	
	public double pareto_shifted(double a, double b, double c, int rng)
	{
	  //  if (a==0) throw exception 

	    double u_pow = Math.pow(1.0 - genk_dblrand(rng), 1.0 / a);
	    return b / u_pow - c;
	}

	public double uniform(double a, double b)
	{
	    return a + genk_dblrand(0) * (b-a);
	}

	public double truncnormal(double m, double d, int rng)
	{
	    double res;
	    do {
	         res = normal(m,d,rng);
	    } while(res<0);

	    return res;
	}

	public double normal(double m, double d, int rng)
	{
	    double U = 1.0 - genk_dblrand(rng);
	    double V = 1.0 - genk_dblrand(rng);
	    return m + d * Math.sqrt(-2.0*Math.log(U)) * Math.cos(PI*2*V);
	}
	
	public double genk_dblrand(int k)  {return Math.random();
	/**
	 * Produces a random double in the range [0,1) using generator k.
	 */
	}
	
	public double residualLifetime(double mean)
	{
	    return shiftedPareto(2, betaByMean(mean));
	}

	public double individualLifetime(double mean)
	{
	    return shiftedPareto(3, betaByMean(mean));
	}

}
