/**
 * 
 * @author giang.ngo_hoang@inria.fr - logNet team 2011 - INRIA
 *         Sophia-Antipolis - France
 */

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;

public class Configuration {
	public int numberOfChordNets;
	public int numberOfKadNets;
	public int percentageOfSynnapse;
	public int numberOfNodePerOverlay;
	public int numberOfSynapse;
	public int numberOfSynapsePerCreationThread;
	public int numberOfAdditionalSynapse;
	public int numberOfDataNodePerOverlay;
	public int numberOfnodes;
	public int synDegree;
	public double lifetimeMean;
	public double deadtimeMean; 
	public double initPhaseCreationInterval;
	public double transitionTime;
	public double measurementTime;
	public double getInterval;
	public double putInterval;
	public double putTime;
	public double initTime;
	public double simulationTime;
	public double startTime = 0;
	public boolean churn;
	
	public Configuration(String filename)
	{
		String delimiter = "=";
		String[] temp;		
		String strLine;		
		try{
			  FileInputStream fstream = new FileInputStream(filename);
			  DataInputStream in = new DataInputStream(fstream);
			  BufferedReader br = new BufferedReader(new InputStreamReader(in));			
			
			  while (((strLine = br.readLine()) != null) && !strLine.equals("#"))  {
				  temp = strLine.split(delimiter);
				  if (temp[0].equals("numberOfChordNets"))
					  numberOfChordNets = Integer.parseInt(temp[1]);
				  else if (temp[0].equals("numberOfKadNets"))
					  numberOfKadNets = Integer.parseInt(temp[1]);
				  else if (temp[0].equals("percentageOfSynnapse"))
					  percentageOfSynnapse = Integer.parseInt(temp[1]);
				  else if (temp[0].equals("numberOfNodesPerOverlay"))
					  numberOfNodePerOverlay = Integer.parseInt(temp[1]);
				  else if (temp[0].equals("synDegree"))
					  synDegree = Integer.parseInt(temp[1]);
				  else if (temp[0].equals("lifetimeMean"))
					  lifetimeMean = Double.parseDouble(temp[1]);
				  else if (temp[0].equals("deadtimeMean"))
					  deadtimeMean = Double.parseDouble(temp[1]);
				  else if (temp[0].equals("initPhaseCreationInterval"))
					  initPhaseCreationInterval = Double.parseDouble(temp[1]);
				  else if (temp[0].equals("transitionTime"))
					  transitionTime = Double.parseDouble(temp[1]);
				  else if (temp[0].equals("measurementTime"))
					  measurementTime = Double.parseDouble(temp[1]) * lifetimeMean;
				  else if (temp[0].equals("getInterval"))
					  getInterval = Double.parseDouble(temp[1]);
				  else if (temp[0].equals("putInterval"))
					  putInterval = Double.parseDouble(temp[1]);
				  else if (temp[0].equals("churn"))
				  {	
					if (temp[1].equals("true"))
						churn = true;
					else 
						churn = false;
				  }
				  
			  }	
			  in.close();
			  
			  					  
		}catch (Exception e){ 
			System.err.println("Error:" + e.getMessage()); 
		}

		
		
		numberOfnodes = (numberOfChordNets + numberOfKadNets)*numberOfNodePerOverlay;	
		
		numberOfSynapse = Math.round(numberOfnodes*percentageOfSynnapse/100);
		
		//numberOfSynapseNodePerOverlay = Math.round(numberOfNodesPerOverlay* percentageOfSynnapse/100); 
		numberOfDataNodePerOverlay = numberOfNodePerOverlay - 
			Math.round(numberOfNodePerOverlay* percentageOfSynnapse/100);
		
		
		if (synDegree > (numberOfChordNets + numberOfKadNets)) 
			synDegree = numberOfChordNets + numberOfKadNets;
		
		
		numberOfSynapsePerCreationThread = numberOfSynapse*2/(numberOfChordNets + numberOfKadNets); 
		
		numberOfAdditionalSynapse = numberOfSynapse - 
				numberOfSynapsePerCreationThread * (numberOfChordNets + numberOfKadNets)/2;
		
		
		if (numberOfAdditionalSynapse > (numberOfChordNets + numberOfKadNets)/4)
			numberOfSynapsePerCreationThread+=1;
		
		if (((numberOfChordNets == 0) && (percentageOfSynnapse == 0)) || 
		((numberOfKadNets == 0) && (percentageOfSynnapse == 0)))
		{
			numberOfDataNodePerOverlay = numberOfDataNodePerOverlay/20;
		}
		
		
		
		if (numberOfSynapsePerCreationThread > numberOfDataNodePerOverlay)
			initTime = numberOfSynapsePerCreationThread * initPhaseCreationInterval;
		else 
			initTime = numberOfDataNodePerOverlay *initPhaseCreationInterval;
		
		initTime = initTime*1.1; // to ensure all overlays are created
		
		//putTime = (numberOfnodes + numberOfSynapse*(synDegree-1))* putInterval;
		
		putTime = 20;
		
		simulationTime = initTime + transitionTime+ putTime + measurementTime;

		printOut();
	}
	

	public void printOut()
	{
		int numberOfPlugins =  numberOfSynapse * synDegree;
		int numberOfDataNodes = numberOfDataNodePerOverlay*(numberOfChordNets + numberOfKadNets);
		int simulationTimeInMin = (int)simulationTime/60 + 5;
		
		System.out.println("--------------------------------------");
		System.out.println("numberOfChordNets:numberOfKadNets:percentageOfSynnapse" +
				":numberOfNodePerOverlay:numberOfDataNodes:numberOfSynapse:numberOfPlugins:" +
				"numberOfSynapsePerCreationThread"+ numberOfChordNets+":"+numberOfKadNets+
				":"+percentageOfSynnapse +":"+numberOfNodePerOverlay+":"+numberOfDataNodes
				+":"+numberOfSynapse+":"+numberOfPlugins+":"+numberOfSynapsePerCreationThread);
		
		System.out.println("lifetimeMean:deadtimeMean:initPhaseCreationInterval:initTime" +
				":transitionTime:putTime:measurementTime:simulationTime(in minute)"+lifetimeMean+":"+
				deadtimeMean+":"+initPhaseCreationInterval+":"+	initTime+":"+transitionTime+
				":"+putTime+":"+measurementTime+":"+simulationTime+"("+simulationTimeInMin+")");
		System.out.println("--------------------------------------");	
	}
	public static void main(String[] args)
	{
		new Configuration("experiment.conf");	
	}
}